/*
 * epgsearch.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include <vdr/plugin.h>
#include <vdr/status.h>
#include <vdr/epg.h>
#include "menu_event.h"
#include "menu_edittimer.h"
#include "menu_main.h"
#include "menu_whatson.h"
#include "i18n.h"
#include "epgsearchsetup.h"
#include "epgsearchcfg.h"
#include "epgsearchext.h"
#include "menu_searchresults.h"

static const char *VERSION        = "0.0.2";
static const char *DESCRIPTION    = "search the EPG for repeatings and more";
static const char *MAINMENUENTRY  = "EPG search";

class cPluginEpgsearch : public cPlugin {
private:
  // Add any member variables or functions you may need here.
public:
  cPluginEpgsearch(void);
  virtual ~cPluginEpgsearch();
  virtual const char *Version(void) { return VERSION; }
  virtual const char *Description(void) { return tr(DESCRIPTION); }
  virtual const char *CommandLineHelp(void);
  virtual bool ProcessArgs(int argc, char *argv[]);
  virtual bool Initialize(void);
  virtual bool Start(void);
  virtual void Housekeeping(void);
  virtual const char *MainMenuEntry(void) {  return (EPGSearchConfig.hidemenu ? NULL : tr(MAINMENUENTRY)); }
  virtual cOsdObject *MainMenuAction(void);
  virtual cMenuSetupPage *SetupMenu(void);
  virtual bool SetupParse(const char *Name, const char *Value);
  };

cPluginEpgsearch::cPluginEpgsearch(void)
{
  // Initialize any member variables here.
  // DON'T DO ANYTHING ELSE THAT MAY HAVE SIDE EFFECTS, REQUIRE GLOBAL
  // VDR OBJECTS TO EXIST OR PRODUCE ANY OUTPUT!
}

cPluginEpgsearch::~cPluginEpgsearch()
{
  // Clean up after yourself!
}

const char *cPluginEpgsearch::CommandLineHelp(void)
{
  // Return a string that describes all known command line options.
  return NULL;
}

bool cPluginEpgsearch::ProcessArgs(int argc, char *argv[])
{
  // Implement command line argument processing here if applicable.
  return true;
}

bool cPluginEpgsearch::Initialize(void)
{
  // Initialize any background activities the plugin shall perform.
  RegisterI18n(Phrases);
  return true;
}

bool cPluginEpgsearch::Start(void)
{
  // Start any background activities the plugin shall perform.
  return true;
}

void cPluginEpgsearch::Housekeeping(void)
{
  // Perform any cleanup or other regular tasks.
}

cOsdObject *cPluginEpgsearch::MainMenuAction(void)
{
  // Perform the action when selected from the main VDR menu.
  return new cMenuSearchMain();
}

cMenuSetupPage *cPluginEpgsearch::SetupMenu(void)
{
  // Return a setup menu in case the plugin supports one.
  return new cMenuEPGSearchSetup;
}

bool cPluginEpgsearch::SetupParse(const char *Name, const char *Value)
{
  // Parse your own setup parameters and store their values.
  if      (!strcasecmp(Name, "HideMenu"))       EPGSearchConfig.hidemenu       = atoi(Value);
  if      (!strcasecmp(Name, "RedKeyMode"))       EPGSearchConfig.redkeymode       = atoi(Value);
  return true;
}

VDRPLUGINCREATOR(cPluginEpgsearch); // Don't touch this!
