#include "epgsearchext.h"
#include <vdr/tools.h>
#include "menu_searchresults.h"

const char *SearchModes[3];
int     q_manual_update; // data has change --> question for manual update
cSearchExts SearchExts;

// -- cSearchExt -----------------------------------------------------------------

char *cSearchExt::buffer = NULL;

cSearchExt::cSearchExt(void)
{
  *search = 0;
  options = 1;
  useTime = false;
  startTime = 0000;
  stopTime = 2359;
  useChannel = false;
  channel = Channels.GetByNumber(cDevice::CurrentChannel());
  useCase = false;
  mode = 0;
  useTitle = true;
  useSubtitle = true;
  useDescription = true;
  buffer = NULL;
}

cSearchExt::~cSearchExt(void)
{
  if (buffer) {
    free(buffer);
    buffer = NULL;
  }
}

cSearchExt& cSearchExt::operator= (const cSearchExt &SearchExt)
{
  memcpy(this, &SearchExt, sizeof(*this));
  return *this;
}

bool cSearchExt::operator< (const cListObject &ListObject)
{
  cSearchExt *SE = (cSearchExt *)&ListObject;
  return strcasecmp(search, SE->search) < 0;
}

const char *cSearchExt::ToText(void)
{
  char tmp_Start[5] = "";
  char tmp_Stop[5] = "";

  free(buffer);
  strreplace(search, ':', '|');

  if (useTime)
  {
    sprintf(tmp_Start, "%04d", startTime);
    sprintf(tmp_Stop, "%04d", stopTime);
  }
  asprintf(&buffer, "%s:%d:%s:%s:%d:%d:%d:%d:%d:%d:%d\n",
                    search,
                    useTime,
                    tmp_Start,
                    tmp_Stop,
		    useChannel,
                    useChannel ? channel->Number() : 0,
  	            useCase,
                    mode,
                    useTitle,
                    useSubtitle,
                    useDescription);


  strreplace(search, '|', ':');


  return buffer;
}

bool cSearchExt::Parse(const char *s)
{
  char *line;
  char *pos;
  char *pos_next;
  int parameter = 1;
  int valuelen;
  char value[MaxFileName];
  tChannelID cid;


  pos = line = strdup(s);
  pos_next = pos + strlen(pos);
  if (*pos_next == '\n') *pos_next = 0;
  while (*pos) {
    while (*pos == ' ') pos++;
    if (*pos) {
      if (*pos != ':') {
        pos_next = strchr(pos, ':');
        if (!pos_next)
          pos_next = pos + strlen(pos);
        valuelen = pos_next - pos + 1;
        if (valuelen > MaxFileName) valuelen = MaxFileName;
        strn0cpy(value, pos, valuelen);
        pos = pos_next;
        switch (parameter) {
          case 1:  strcpy(search, value);
                   break;
          case 2:  useTime = atoi(value);
                   break;
          case 3:  startTime = atoi(value);
		   break;
          case 4:  stopTime = atoi(value);
		   break;
          case 5:  useChannel = atoi(value);
                   break;
          case 6:  if (!useChannel)
			channel = NULL;
		   else
		   {
		   	cid = tChannelID::FromString(value);
                   	channel = cid.Valid() ? Channels.GetByChannelID(cid, true) : Channels.GetByNumber(atoi(value));
                   	if (!channel && atoi(value)>0) {
                       		free(line);
                       		return false;
                     	}
		   }
                   break;
          case 7:  useCase = atoi(value);
                   break;
          case 8:  mode = atoi(value);
                   break;
          case 9: useTitle = atoi(value);
                   break;
          case 10: useSubtitle = atoi(value);
                   break;
          case 11: useDescription = atoi(value);
                   break;
        } //switch
      }
      parameter++;
    }
    if (*pos) pos++;
  } //while

  strreplace(search, '|', ':');

  free(line);
  return (parameter >= 11) ? true : false;
}

bool cSearchExt::Save(FILE *f)
{
  return fprintf(f, ToText()) > 0;
}

// -- cSearchExts ----------------------------------------------------------------

cSearchExts::cSearchExts(void)
{
}


// --- cMenuEditSearchExt --------------------------------------------------------


cMenuEditSearchExt::cMenuEditSearchExt(cSearchExt *SearchExt, bool New)
:cOsdMenu(tr("Edit search"),30)
{
  searchExt = SearchExt;
  addIfConfirmed = New;
  if (searchExt) 
  {
     data = *searchExt;
     useTime = data.useTime;
     if (data.useChannel)
        channel = data.channel->Number();
     else
        channel = cDevice::CurrentChannel();
     Set();
  }
}

void cMenuEditSearchExt::Set()
{
  int current = Current();
  Clear();
  Add(new cMenuEditStrItem( tr("Search term"), data.search, sizeof(data.search), tr(FileNameChars)));
  Add(new cMenuEditStraItem(tr("Search mode"),     &data.mode, 3, SearchModes));
  Add(new cMenuEditBoolItem( tr("Match case"), &data.useCase, tr("no"), tr("yes")));
  Add(new cMenuEditBoolItem( tr("Use Title"), &data.useTitle, tr("no"), tr("yes")));
  Add(new cMenuEditBoolItem( tr("Use Subtitle"), &data.useSubtitle, tr("no"), tr("yes")));
  Add(new cMenuEditBoolItem( tr("Use Description"), &data.useDescription, tr("no"), tr("yes")));
  Add(new cMenuEditBoolItem( tr("Use Channel"), &data.useChannel, tr("no"), tr("yes")));
  if (data.useChannel)
  	Add(new cMenuEditChanItem(tr("Channel"),      &channel));
  Add(new cMenuEditBoolItem( tr("Use Time"), &useTime, tr("no"), tr("yes")));
  if (useTime == true)
  {
     Add(new cMenuEditTimeItem(tr("Start after"),        &data.startTime));
     Add(new cMenuEditTimeItem(tr("Start before"),         &data.stopTime));
  }
  SetCurrent(Get(current));
}

cMenuEditSearchExt::~cMenuEditSearchExt()
{
  if (searchExt && addIfConfirmed)
     delete searchExt; // apparently it wasn't confirmed
}


eOSState cMenuEditSearchExt::ProcessKey(eKeys Key)
{
  int iTemp_useTime = useTime;
  int iTemp_useChannel = data.useChannel;

  eOSState state = cOsdMenu::ProcessKey(Key);

  if (useTime != iTemp_useTime ||
      iTemp_useChannel != data.useChannel)
  {
     Set();
     Display();
  }

  if (state == osUnknown) {
    switch (Key) {
      case kOk:       if (!strlen(data.search)) {
                        Skins.Message(mtError, tr("Error$Missing search text!"));
                        break;
                      }
                      if (data.useChannel) {
                        cChannel *ch = Channels.GetByNumber(channel);
                        if (ch)
                          data.channel = ch;
                        else {
                          Skins.Message(mtError, tr("*** Invalid Channel ***"));
                          break;
                        }
                      }
                      if ((int(strlen(data.search)) < 3) && !Interface->Confirm(tr("Edit$Search text to small - use anyway?")))
                        break;
                      data.useTime = useTime;
                     if (searchExt) {
                        if (memcmp(searchExt, &data, sizeof(data)) != 0) { // no char* used
                          *searchExt = data;
                          if (addIfConfirmed)
                            SearchExts.Add(searchExt);
                          SearchExts.Save();
                          addIfConfirmed = false;
                        }
                      }
                      return osBack;
      case kBlue:   
      case kRed:
      case kGreen:
      case kYellow: state = osContinue;
      default: break;
    }
  }
  return state;
}



// --- cMenuSearchExtItem ----------------------------------------------------------

class cMenuSearchExtItem : public cOsdItem {
  private:
public:
  cSearchExt* searchExt;
  cMenuSearchExtItem(cSearchExt* SearchExt);
  void Set(void);
  };

cMenuSearchExtItem::cMenuSearchExtItem(cSearchExt* SearchExt)
{
  searchExt = SearchExt;
  Set();
}

void cMenuSearchExtItem::Set(void)
{
  char *buffer = NULL;
  char *tmp[]  = { "", "", "", "" };

  asprintf(&tmp[0], "%s", searchExt->search);

  asprintf(&tmp[1], "\t%.7s", (searchExt->useChannel?searchExt->channel->Name():""));

  if (searchExt->useTime)
  {
	  asprintf(&tmp[2], "\t%02d:%02d", searchExt->startTime / 100, searchExt->startTime % 100);
	  asprintf(&tmp[3], "\t%02d:%02d", searchExt->stopTime / 100, searchExt->stopTime % 100);
  }
  else
  {
	  asprintf(&tmp[2], "\t--:--");
	  asprintf(&tmp[3], "\t--:--");
  }
	
  asprintf(&buffer, "%s%s%s%s", tmp[0], tmp[1], tmp[2], tmp[3]);
  SetText(buffer, false);
}

// --- cMenuEPGSearchExt ----------------------------------------------------------
cMenuEPGSearchExt::cMenuEPGSearchExt(void)
:cOsdMenu(tr("Search"), 20, 6, 6, 6)
{
  SearchModes[0] =      tr("Phrase");
  SearchModes[1] =      tr("all words");
  SearchModes[2] =      tr("one word");

  SearchExts.Load(AddDirectory( cPlugin::ConfigDirectory(), "epgsearch.conf"));
  cSearchExt *SearchExt = SearchExts.First();
  while (SearchExt) {
    Add(new cMenuSearchExtItem(SearchExt));
    SearchExt = SearchExts.Next(SearchExt);
  }

  SetHelp(tr("Edit"), tr("New"), tr("Delete"), tr("Start"));
  Display();
}

cSearchExt *cMenuEPGSearchExt::CurrentSearchExt(void)
{
    cMenuSearchExtItem *item = (cMenuSearchExtItem *)Get(Current());
    return item ? item->searchExt : NULL;
}


eOSState cMenuEPGSearchExt::New(void)
{
  if (HasSubMenu())
    return osContinue;
  return AddSubMenu(new cMenuEditSearchExt(new cSearchExt, true));
}

eOSState cMenuEPGSearchExt::Delete(void)
{
  cSearchExt *curSearchExt = CurrentSearchExt();
  if (curSearchExt) {
    if (Interface->Confirm(tr("Edit$Delete Search?"))) {
      SearchExts.Del(curSearchExt);
      SearchExts.Save();
      cOsdMenu::Del(Current());
      Display();
    }
  }
  return osContinue;
}

eOSState cMenuEPGSearchExt::Search(void)
{
	return AddSubMenu(new cMenuSearchResults(CurrentSearchExt()));
}

eOSState cMenuEPGSearchExt::ProcessKey(eKeys Key)
{
  int SearchNumber = HasSubMenu() ? Count() : -1;
  eOSState state = cOsdMenu::ProcessKey(Key);
  if (state == osUnknown) {
    switch (Key) {
	case kOk:
	case kBlue:
		state = Search();
		break;
	case kRed:
	  if (HasSubMenu())
	    return osContinue;
	  state = AddSubMenu(new cMenuEditSearchExt(CurrentSearchExt()));
		break;
     	case kGreen: state = New(); break;
        case kYellow: state = Delete(); break;

      default: break;
    }
  }
  if (SearchNumber >= 0 && !HasSubMenu() && SearchExts.Get(SearchNumber)) {
     // a newly created search was confirmed with Ok
     Add(new cMenuSearchExtItem(SearchExts.Get(SearchNumber)));
     Display();
     }

  return state;
}


