/*
 * menu_commands.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include "menu_commands.h"
#include <vdr/interface.h>
#include <vdr/menu.h>
#include <vdr/plugin.h>
#include "menu_searchresults.h"
#include "menu_edittimer.h"

// --- cMenuSearchCommands ---------------------------------------------------------

cMenuSearchCommands::cMenuSearchCommands(const char *Title, const cEvent* Event)
:cOsdMenu(Title)
{
  SetHasHotkeys();
  commands.Load(AddDirectory(cPlugin::ConfigDirectory(), "epgsearchcmds.conf"), true);
  event = Event;
  Add(new cOsdItem(hk(tr("Repeatings"))));
  Add(new cOsdItem(hk(tr("Record"))));
  for (cCommand *command = commands.First(); command; command = commands.Next(command))
      Add(new cOsdItem(hk(command->Title())));
}

cMenuSearchCommands::~cMenuSearchCommands()
{
}

eOSState cMenuSearchCommands::Record(void)
{
     if (!event) return osContinue;
     cTimer *timer = new cTimer(event);
     cTimer *t = Timers.GetTimer(timer);
     if (t) {
        delete timer;
        timer = t;
        }
     return AddSubMenu(new cMenuEditTimer(timer, !t));
}

eOSState cMenuSearchCommands::Execute(void)
{
  int current = Current();
  if (current <= 1)
  {
     if (current == 0)
     	return AddSubMenu(new cMenuSearchResults(event->Title()));
     if (current == 1)
     	return Record(); 
  }
  cCommand *command = commands.Get(current-2);
  if (command) {
     char *buffer = NULL;
     bool confirmed = true;
     if (command->Confirm()) {
        asprintf(&buffer, "%s?", command->Title());
        confirmed = Interface->Confirm(buffer);
        free(buffer);
        }
     if (confirmed) {
        asprintf(&buffer, "%s...", command->Title());
        Skins.Message(mtStatus, buffer);
        free(buffer);

        asprintf(&buffer, "\"%s\" %ld %ld %d", event->Title(), event->StartTime(), event->EndTime(), Channels.GetByChannelID(event->ChannelID())->Number());
        const char *Result = command->Execute(buffer);
        free(buffer);
        Skins.Message(mtStatus, NULL);
        if (Result)
           return AddSubMenu(new cMenuText(command->Title(), Result, fontFix));
        return osEnd;
        }
     }
  return osContinue;
}

eOSState cMenuSearchCommands::ProcessKey(eKeys Key)
{
  eOSState state = cOsdMenu::ProcessKey(Key);

  if (state == osUnknown) {
     switch (Key) {
       case k0:
     		return AddSubMenu(new cMenuSearchResults(event->Title()));
       case kOk:  return Execute();
       default:   break;
       }
     }
  return state;
}
