/*
 * menu_main.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */
#include "menu_main.h"
#include "menu_whatson.h"
#include "menu_edittimer.h"
#include "epgsearchext.h"
#include "menu_event.h"
#include "menu_searchresults.h"
#include "menu_commands.h"
#include "epgsearchcfg.h"

cMenuScheduleItem::cMenuScheduleItem(const cEvent *Event)
{
  event = Event;
  char *buffer = NULL;
  int TimerMatch;
  char t = Timers.GetMatch(Event, &TimerMatch) ? (TimerMatch == tmFull) ? 'T' : 't' : ' ';
  char v = event->Vps() && (event->Vps() - event->StartTime()) ? 'V' : ' ';
  char r = event->IsRunning() ? '*' : ' ';
  asprintf(&buffer, "%.*s\t%s\t%c%c%c\t%s", 6, event->GetDateString(), event->GetTimeString(), t, v, r, event->Title());
  SetText(buffer, false);
}

// --- cMenuSearchMain ---------------------------------------------------------

cMenuSearchMain::cMenuSearchMain(void)
:cOsdMenu("", 7, 6, 4)
{
  now = next = false;
  otherChannel = 0;
  cChannel *channel = Channels.GetByNumber(cDevice::CurrentChannel());
  if (channel) {
     cMenuWhatsOnSearch::SetCurrentChannel(channel->Number());
     schedules = cSchedules::Schedules(schedulesLock);
     PrepareSchedule(channel);
     SetHelp((EPGSearchConfig.redkeymode==0?tr("Record"):tr("Button$Commands")), tr("Now"), tr("Next"), tr("Button$Search"));
     }
}

cMenuSearchMain::~cMenuSearchMain()
{
  cMenuWhatsOnSearch::ScheduleEvent(); // makes sure any posted data is cleared
}

void cMenuSearchMain::PrepareSchedule(cChannel *Channel)
{
  Clear();
  char *buffer = NULL;
  asprintf(&buffer, tr("Schedule - %s"), Channel->Name());
  SetTitle(buffer);
  free(buffer);
  if (schedules) {
     const cSchedule *Schedule = schedules->GetSchedule(Channel->GetChannelID());
     if (Schedule) {
        const cEvent *PresentEvent = Schedule->GetPresentEvent(Channel->Number() == cDevice::CurrentChannel());
        time_t now = time(NULL) - Setup.EPGLinger * 60;
        for (const cEvent *Event = Schedule->Events()->First(); Event; Event = Schedule->Events()->Next(Event)) {
            if (Event->EndTime() > now || Event == PresentEvent)
               Add(new cMenuScheduleItem(Event), Event == PresentEvent);
            }
        }
     }
}

eOSState cMenuSearchMain::Record(void)
{
  cMenuScheduleItem *item = (cMenuScheduleItem *)Get(Current());
  if (item) {
     cTimer *timer = new cTimer(item->event);
     cTimer *t = Timers.GetTimer(timer);
     if (t) {
        delete timer;
        timer = t;
        }
     return AddSubMenu(new cMenuEditTimer(timer, !t));
     }
  return osContinue;
}

eOSState cMenuSearchMain::Switch(void)
{
  cMenuWhatsOnSearchItem *item = (cMenuWhatsOnSearchItem *)Get(Current());
  if (item) {
     cChannel *channel = Channels.GetByChannelID(item->event->ChannelID(), true);
     if (channel && cDevice::PrimaryDevice()->SwitchChannel(channel, true))
        return osEnd;
     }
  Skins.Message(mtInfo, tr("Can't switch channel!")); 
  return osContinue;
}

eOSState cMenuSearchMain::ExtendedSearch(void)
{
    return AddSubMenu(new cMenuEPGSearchExt());
}

eOSState cMenuSearchMain::Commands(eKeys Key)
{
  if (HasSubMenu() || Count() == 0)
     return osContinue;
  cMenuWhatsOnSearchItem *mi = (cMenuWhatsOnSearchItem *)Get(Current());
  if (mi && mi->event) {
        cMenuSearchCommands *menu;
        eOSState state = AddSubMenu(menu = new cMenuSearchCommands(tr("EPG Commands"), mi->event));
        if (Key != kNone)
           state = menu->ProcessKey(Key);
        return state;
     }
  return osContinue;
}

eOSState cMenuSearchMain::ProcessKey(eKeys Key)
{
  eOSState state = cOsdMenu::ProcessKey(Key);

  if (state == osUnknown) {
     switch (Key) {
       case kRecord:
       case kRed:
		if(HasSubMenu()) {
                        state = Record();
                        break;
                     } 
      		if (Count())
		{
			if (EPGSearchConfig.redkeymode==0)
				state = Record();
			else
			{
				cMenuWhatsOnSearchItem *mi = (cMenuWhatsOnSearchItem *)Get(Current());
	                        if (mi) {
        	                  if (mi->event) {
				     return AddSubMenu(new cMenuSearchCommands(tr("EPG Commands"),mi->event));
					}
				}
			}
		}
		break;
       case k0:
      		if (Count())
		{
			cMenuWhatsOnSearchItem *mi = (cMenuWhatsOnSearchItem *)Get(Current());
                        if (mi) 
                        {
       	                  if (mi->event) 
			     return AddSubMenu(new cMenuSearchCommands(tr("EPG Commands"),mi->event));
			}
		}
		break;
       case k1...k9: return Commands(Key);
       case kGreen:  if (schedules) {
                        if (!now && !next) {
                           int ChannelNr = 0;
                           if (Count()) {
                              cChannel *channel = Channels.GetByChannelID(((cMenuScheduleItem *)Get(Current()))->event->ChannelID(), true);
                              if (channel)
                                 ChannelNr = channel->Number();
                              }
                           now = true;
                           return AddSubMenu(new cMenuWhatsOnSearch(schedules, true, ChannelNr));
                           }
                        now = !now;
                        next = !next;
                        return AddSubMenu(new cMenuWhatsOnSearch(schedules, now, cMenuWhatsOnSearch::CurrentChannel()));
                        }
       case kYellow: if (schedules)
                        return AddSubMenu(new cMenuWhatsOnSearch(schedules, false, cMenuWhatsOnSearch::CurrentChannel()));
                     break;
       case kBlue:   if (HasSubMenu())
			return Switch();
		     return ExtendedSearch();
                     break;
       case kOk:     if (Count())
                        return AddSubMenu(new cMenuEventSearch(((cMenuScheduleItem *)Get(Current()))->event, otherChannel));
                     break;
       default:      break;
       }
     }
  else if (!HasSubMenu()) {
     now = next = false;
     const cEvent *ei = cMenuWhatsOnSearch::ScheduleEvent();
     if (ei) {
        cChannel *channel = Channels.GetByChannelID(ei->ChannelID(), true);
        if (channel) {
           PrepareSchedule(channel);
           if (channel->Number() != cDevice::CurrentChannel()) {
              otherChannel = channel->Number();
     	      SetHelp((EPGSearchConfig.redkeymode==0?tr("Record"):tr("Button$Commands")), tr("Now"), tr("Next"), tr("Button$Search"));
              }
           Display();
           }
        }
     }
  return state;
}

