/*
 * menu_searchresults.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include <ctype.h>
#include "menu_searchresults.h"
#include "menu_edittimer.h"
#include "menu_event.h"


static int CompareEventTime(const void *p1, const void *p2)
{
  return (int)((*(cEvent **)p1)->StartTime() - (*(cEvent **)p2)->StartTime());
}

static int CompareEventChannel(const void *p1, const void *p2)
{
  return (int)( Channels.GetByChannelID((*(const cEvent **)p1)->ChannelID())->Number() - Channels.GetByChannelID((*(const cEvent **)p2)->ChannelID())->Number());
}

void ToLower(char* szText)
{
	if (!szText)
		return;
	for (int loop = 0; szText[loop] !=0; loop++)
        szText[loop] = tolower(szText[loop]);
}



// --- cMenuSearchResultsItem -------------------------------------------------------

cMenuSearchResultsItem::cMenuSearchResultsItem(const cEvent *EventInfo)
{
  eventInfo = EventInfo;
  char *buffer = NULL;
  cChannel *channel = Channels.GetByChannelID(eventInfo->ChannelID(), true);
  if(eventInfo->ShortText())
     asprintf(&buffer, "%.*s\t%.*s\t%.*s\t%s~%s", 10, channel ? channel->Name() : "???",
                        6, eventInfo->GetDateString(), 5, eventInfo->GetTimeString(), eventInfo->Title(), eventInfo->ShortText());
  else
     asprintf(&buffer, "%.*s\t%.*s\t%.*s\t%s", 10, channel ? channel->Name() : "???",
                        6, eventInfo->GetDateString(), 5, eventInfo->GetTimeString(), eventInfo->Title());
  SetText(buffer, false);
}

// --- cMenuSearchResults -------------------------------------------------------

const cEvent *cMenuSearchResults::scheduleEventInfo = NULL;
int cMenuSearchResults::currentChannel = 0;

cMenuSearchResults::cMenuSearchResults(const char *p2)
:cOsdMenu(tr("additional starting times"), 13, 6, 6)
{
  asprintf(&m_pSearchString, "%s", p2);
  searchExt = NULL;
  m_bSort = true;

  BuildList();
}

cMenuSearchResults::cMenuSearchResults(cSearchExt* SearchExt)
:cOsdMenu(tr("Search results"), 12, 6, 6)
{
  searchExt = SearchExt;
  m_pSearchString = NULL;
  m_bSort = true;

  BuildList();
}

cEvent * cMenuSearchResults::GetEventByTitle(const cSchedule *schedules, const char *p2, const cEvent *Start) 
{
  if(!p2) return NULL;

  cEvent *pe = NULL;
  cEvent *p1=schedules->Events()->First();
// first we have to search for the Start item
  if(Start) {
     cEvent *p;
     for (p = schedules->Events()->First(); p && p!=Start; p = schedules->Events()->Next(p));
     if(p)
        p1=schedules->Events()->Next(p);
     }

  time_t tNow=time(NULL);
  for (cEvent *p = p1; p; p = schedules->Events()->Next(p)) {
     if(!p) {
        break;
     }
     if (tNow < p->EndTime()) {
        if(p->Title() && !strcasecmp(p2, p->Title())) {
           pe=p;
           break;
        }
     }
  }
  return pe;
}

cEvent * cMenuSearchResults::GetEventBySearchExt(const cSchedule *schedules, cSearchExt* SearchExt, const cEvent *Start) 
{
  if(!SearchExt) return NULL;

  cEvent *pe = NULL;
  cEvent *p1=schedules->Events()->First();
  // first we have to search for the Start item
  if(Start) {
     cEvent *p;
     for (p = schedules->Events()->First(); p && p!=Start; p = schedules->Events()->Next(p));
     if(p)
        p1=schedules->Events()->Next(p);
     }

  time_t tNow=time(NULL);
  char* szTest = NULL;
  char* searchText = NULL;
  asprintf(&searchText, "%s", SearchExt->search);

  int searchStart = 0, searchStop = 0;
  if (SearchExt->useTime)
  {
	searchStart = SearchExt->startTime;
	searchStop = SearchExt->stopTime;
	if (searchStop < searchStart)
		searchStop += 2400;
  }

  for (cEvent *p = p1; p; p = schedules->Events()->Next(p))
  {
     if(!p) 
     {
        break;
     }
     if (szTest)
     {
	free(szTest);
	szTest = NULL;
     }
     asprintf(&szTest, "%s%s%s", (SearchExt->useTitle?p->Title():""),
				 (SearchExt->useSubtitle?p->ShortText():""),
				 (SearchExt->useDescription?p->Description():""));     
	
     if (tNow < p->EndTime()) 
     {
        if (!SearchExt->useCase)
        {
		ToLower(szTest);
                ToLower(searchText);
        }
	
       	bool bTesting=false;
        if (SearchExt->mode == 0 && !strstr(szTest, searchText)) 
		continue;
        else
		bTesting = true;
        if (SearchExt->mode > 0)
        { 
		char *pstrSearchToken;
        	bool bFirst=true;
	        char *pstrSearch=strdup(searchText);
       		pstrSearchToken=strtok(pstrSearch, " ,;|");
        	while(pstrSearchToken) {
           		if(szTest && strstr(szTest, pstrSearchToken)) { 
              			if(SearchExt->mode==1) { // means AND
                 			if(bFirst) {
                    				bTesting=true;
                    				bFirst=false;
                 			} else
                    			bTesting&=true;
              			}
              			else
                 			bTesting|=true;
           		} else {// not found!!
              			if(SearchExt->mode==1) { // means AND
                 			bTesting=false;
                 			bFirst=false;
              			}
           		}
          		pstrSearchToken=strtok(NULL, " ,;|");
		}
		free(pstrSearch);

        }
        if(!bTesting)
		continue;
       

        if (SearchExt->useTime)
        {
		time_t tEvent = p->StartTime();
		struct tm tmEvent = *localtime(&tEvent);
		int eventStart = tmEvent.tm_hour*100 + tmEvent.tm_min;	
		int eventStart2 = tmEvent.tm_hour*100 + tmEvent.tm_min + 2400;	
		if ((eventStart < searchStart || eventStart > searchStop) &&
		    (eventStart2 < searchStart || eventStart2 > searchStop))
			continue;
        }
	pe=p;
	break;
     }
  }
  if (szTest)
	free(szTest);
  free(searchText);
  return pe;
}

bool cMenuSearchResults::BuildList(void)
{
  cSchedulesLock schedulesLock;
  const cSchedules *schedules;
  schedules = cSchedules::Schedules(schedulesLock);
  if(!schedules) {
     esyslog("cMenuSearchResults::BuildList scheduler is empty!");
     return false;
  }
  const cSchedule *Schedule = schedules->First();
  const cEvent **pArray = NULL;
  const cEvent *pPrevEvent = NULL;
  int num = 0;

  Clear();
  while (Schedule) {
        if (searchExt && searchExt->useChannel && searchExt->channel)
	{
		cChannel* channel = Channels.GetByChannelID(Schedule->ChannelID(),true);
		if (channel && searchExt->channel->Number() != channel->Number())
		{
			Schedule = (const cSchedule *)schedules->Next(Schedule);
			continue;
		}
	}

        pArray = (const cEvent **)realloc(pArray, (num + 1) * sizeof(cEvent *));
        pPrevEvent = NULL;
        do {
	   if (!searchExt)
           	pArray[num] = GetEventByTitle(Schedule, m_pSearchString, pPrevEvent);
	   else
           	pArray[num] = GetEventBySearchExt(Schedule, searchExt, pPrevEvent);
           pPrevEvent = pArray[num];
           if (pArray[num]) {
              cChannel *channel = Channels.GetByChannelID(pArray[num]->ChannelID(), true);
              if (channel) {
                 num++;
                 pArray = (const cEvent **)realloc(pArray, (num + 1) * sizeof(cEvent *));
              }
           }
        } while(pPrevEvent);
        Schedule = (const cSchedule *)schedules->Next(Schedule);
        }

  qsort(pArray, num, sizeof(cEvent *), m_bSort? CompareEventTime: CompareEventChannel);

  for (int a = 0; a < num; a++)
     Add(new cMenuSearchResultsItem(pArray[a]));

  delete pArray;
  
  SetHelp(tr("Record"), m_bSort? tr("by Channel"):tr("by Time"), NULL, NULL);
  Display();
  return true;
}


eOSState cMenuSearchResults::Record(void)
{
  cMenuSearchResultsItem *item = (cMenuSearchResultsItem *)Get(Current());
  if (item) {
     cTimer *timer = new cTimer(item->eventInfo);
     cTimer *t = Timers.GetTimer(timer);
     if (t) {
        delete timer;
        timer = t;
        }
     return AddSubMenu(new cMenuEditTimer(timer, !t));
     }
  return osContinue;
}


eOSState cMenuSearchResults::Switch(void)
{
  cMenuSearchResultsItem *item = (cMenuSearchResultsItem *)Get(Current());
  if (item) {
     cChannel *channel = Channels.GetByChannelID(item->eventInfo->ChannelID(), true);
     if (channel && cDevice::PrimaryDevice()->SwitchChannel(channel, true))
        return osEnd;
     }
  Skins.Message(mtInfo, tr("Can't switch channel!")); 
  return osContinue;
}

eOSState cMenuSearchResults::ProcessKey(eKeys Key)
{
  eOSState state = osContinue;

     switch (Key) {
       case kRecord:
       case kRed:    return Record();
       case k0:      state=osBack; break;
       case kGreen:  {
                        m_bSort=!m_bSort;
			BuildList();
                     }
                     break;
       case kBlue:
		     if (HasSubMenu())
			return Switch();
       case kYellow: break;
       case kOk:     if(HasSubMenu()) {
                        state = cOsdMenu::ProcessKey(Key);
                        break;
                     }
		     if (Count())
		     {
			const cEvent *ei = ((cMenuSearchResultsItem *)Get(Current()))->eventInfo;
     			if (ei) 
			{
				cChannel *channel = Channels.GetByChannelID(ei->ChannelID(), true);
				if (channel)
		               		return AddSubMenu(new cMenuEventSearch(ei, channel->Number() != cDevice::CurrentChannel()));
			}
		     }
                     break;
       default:      
                     state = cOsdMenu::ProcessKey(Key);
     }
  return state;
}


