/*
 * menu_whatson.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */
#include "menu_whatson.h"
#include "epgsearchext.h"
#include "menu_event.h"
#include "menu_edittimer.h"
#include "menu_searchresults.h"
#include "menu_commands.h"
#include "epgsearchcfg.h"

#define CHNUMWIDTH  (numdigits(Channels.MaxNumber()) + 1)

// --- cMenuWhatsOnSearchItem ------------------------------------------------------

cMenuWhatsOnSearchItem::cMenuWhatsOnSearchItem(const cEvent *Event, cChannel *Channel)
{
  event = Event;
  channel = Channel;
  char *buffer = NULL;
  int TimerMatch;
  char t = Timers.GetMatch(Event, &TimerMatch) ? (TimerMatch == tmFull) ? 'T' : 't' : ' ';
  char v = event->Vps() && (event->Vps() - event->StartTime()) ? 'V' : ' ';
  char r = event->IsRunning() ? '*' : ' ';
  asprintf(&buffer, "%d\t%.*s\t%s\t%c%c%c\t%s", channel->Number(), 6, channel->Name(), event->GetTimeString(), t, v, r, event->Title());
  SetText(buffer, false);
}


// --- cMenuWhatsOnSearch ----------------------------------------------------------

int cMenuWhatsOnSearch::currentChannel = 0;
const cEvent *cMenuWhatsOnSearch::scheduleEvent = NULL;

cMenuWhatsOnSearch::cMenuWhatsOnSearch(const cSchedules *Schedules, bool Now, int CurrentChannelNr)
:cOsdMenu(Now ? tr("What's on now?") : tr("What's on next?"), CHNUMWIDTH, 7, 6, 4)
{
  for (cChannel *Channel = Channels.First(); Channel; Channel = Channels.Next(Channel)) {
      if (!Channel->GroupSep()) {
         const cSchedule *Schedule = Schedules->GetSchedule(Channel->GetChannelID());
         if (Schedule) {
            const cEvent *Event = Now ? Schedule->GetPresentEvent() : Schedule->GetFollowingEvent();
            if (Event)
               Add(new cMenuWhatsOnSearchItem(Event, Channel), Channel->Number() == CurrentChannelNr);
            }
         }
      }
  currentChannel = CurrentChannelNr;
  SetHelp((EPGSearchConfig.redkeymode==0?tr("Record"):tr("Button$Commands")), Now ? tr("Next") : tr("Now"), tr("Button$Schedule"), tr("Button$Search"));
}

const cEvent *cMenuWhatsOnSearch::ScheduleEvent(void)
{
  const cEvent *ei = scheduleEvent;
  scheduleEvent = NULL;
  return ei;
}


eOSState cMenuWhatsOnSearch::Switch(void)
{
  cMenuWhatsOnSearchItem *item = (cMenuWhatsOnSearchItem *)Get(Current());
  if (item) {
     cChannel *channel = Channels.GetByChannelID(item->event->ChannelID(), true);
     if (channel && cDevice::PrimaryDevice()->SwitchChannel(channel, true))
        return osEnd;
     }
  Skins.Message(mtInfo, tr("Can't switch channel!")); 
  return osContinue;
}

eOSState cMenuWhatsOnSearch::Record(void)
{
  cMenuWhatsOnSearchItem *item = (cMenuWhatsOnSearchItem *)Get(Current());
  if (item) {
     cTimer *timer = new cTimer(item->event);
     cTimer *t = Timers.GetTimer(timer);
     if (t) {
        delete timer;
        timer = t;
        }
     return AddSubMenu(new cMenuEditTimer(timer, !t));
     }
  return osContinue;
}

eOSState cMenuWhatsOnSearch::Commands(eKeys Key)
{
  if (HasSubMenu() || Count() == 0)
     return osContinue;
  cMenuWhatsOnSearchItem *mi = (cMenuWhatsOnSearchItem *)Get(Current());
  if (mi) {
        cMenuSearchCommands *menu;
        eOSState state = AddSubMenu(menu = new cMenuSearchCommands(tr("EPG Commands"), mi->event));
        if (Key != kNone)
           state = menu->ProcessKey(Key);
        return state;
     }
  return osContinue;
}

eOSState cMenuWhatsOnSearch::ExtendedSearch(void)
{
    return AddSubMenu(new cMenuEPGSearchExt());
}

eOSState cMenuWhatsOnSearch::ProcessKey(eKeys Key)
{
  eOSState state = cOsdMenu::ProcessKey(Key);

  if (state == osUnknown) {
     switch (Key) {
       case kRecord:
       case kRed:
		if(HasSubMenu()) {
                        state = Record();
                        break;
                     } 
      		if (Count())
		{
			if (EPGSearchConfig.redkeymode==0)
				state = Record();
			else
			{
				cMenuWhatsOnSearchItem *mi = (cMenuWhatsOnSearchItem *)Get(Current());
	                        if (mi) {
        	                  if (mi->event) {
				     return AddSubMenu(new cMenuSearchCommands(tr("EPG Commands"),mi->event));
					}
				}
			}
		}
		break;
       case k0:
      		if (Count())
		{
			cMenuWhatsOnSearchItem *mi = (cMenuWhatsOnSearchItem *)Get(Current());
                        if (mi) 
                        {
       	                  if (mi->event) 
			     return AddSubMenu(new cMenuSearchCommands(tr("EPG Commands"),mi->event));
			}
		}
		break;
       case k1...k9: return Commands(Key);
       case kYellow: state = osBack;
                     // continue with kGreen
       case kGreen:  {
                       cMenuWhatsOnSearchItem *mi = (cMenuWhatsOnSearchItem *)Get(Current());
                       if (mi) {
                          scheduleEvent = mi->event;
                          currentChannel = mi->channel->Number();
                          }
                     }
                     break;
       case kBlue:   if (HasSubMenu())
			return Switch();
		     return ExtendedSearch();
		     break;
       case kOk:     if (Count())
		     {
			const cEvent *ei = ((cMenuWhatsOnSearchItem *)Get(Current()))->event;
     			if (ei) 
			{
                           cChannel *channel = Channels.GetByChannelID(ei->ChannelID(), true);
			   if (channel)
                    		return AddSubMenu(new cMenuEventSearch(ei, channel->Number() != cDevice::CurrentChannel()));
			}
		     }	
                     break;
       default:      break;
       }
     }
  return state;
}


